@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-"""\
+"""
 This is a python port of "Goose" orignialy licensed to Gravity.com
 under one or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
@@ -20,39 +19,41 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
-import os
 import glob
+import logging
+import os
 from copy import deepcopy
 
-from langdetect import detect, DetectorFactory
-from langdetect.lang_detect_exception import LangDetectException
-
 import dateutil.parser
 from dateutil.tz import tzutc
+from langdetect import DetectorFactory, detect
+from langdetect.lang_detect_exception import LangDetectException
 
 from goose3.article import Article
-from goose3.utils import URLHelper, RawHelper
-from goose3.text import get_encodings_from_content
+from goose3.cleaners import StandardDocumentCleaner
+from goose3.configuration import Configuration
+from goose3.extractors.authors import AuthorsExtractor
 from goose3.extractors.content import StandardContentExtractor
-from goose3.extractors.videos import VideoExtractor
-from goose3.extractors.title import TitleExtractor
 from goose3.extractors.images import ImageExtractor
 from goose3.extractors.links import LinksExtractor
-from goose3.extractors.tweets import TweetsExtractor
-from goose3.extractors.authors import AuthorsExtractor
-from goose3.extractors.tags import TagsExtractor
-from goose3.extractors.opengraph import OpenGraphExtractor
-from goose3.extractors.publishdate import PublishDateExtractor, TIMEZONE_INFO
-from goose3.extractors.schema import SchemaExtractor
 from goose3.extractors.metas import MetasExtractor
-from goose3.cleaners import StandardDocumentCleaner
-from goose3.outputformatters import StandardOutputFormatter
-
+from goose3.extractors.opengraph import OpenGraphExtractor
+from goose3.extractors.publishdate import TIMEZONE_INFO, PublishDateExtractor
+from goose3.extractors.schema import SchemaExtractor
+from goose3.extractors.tags import TagsExtractor
+from goose3.extractors.title import TitleExtractor
+from goose3.extractors.tweets import TweetsExtractor
+from goose3.extractors.videos import VideoExtractor
 from goose3.network import NetworkFetcher
+from goose3.outputformatters import StandardOutputFormatter
+from goose3.text import get_encodings_from_content
+from goose3.utils import ParsingCandidate, RawHelper, URLHelper
+
+logger = logging.getLogger(__name__)
 
 
-class CrawlCandidate(object):
-    def __init__(self, config, url, raw_html):
+class CrawlCandidate:
+    def __init__(self, config: Configuration, url: str, raw_html: str):
         self.config = config
         # parser
         self.parser = self.config.get_parser()
@@ -60,8 +61,8 @@ class CrawlCandidate(object):
         self.raw_html = raw_html
 
 
-class Crawler(object):
-    def __init__(self, config, fetcher=None):
+class Crawler:
+    def __init__(self, config: Configuration, fetcher=None):
         # config
         self.config = config
         # parser
@@ -118,9 +119,6 @@ class Crawler(object):
         # image extractor
         self.image_extractor = self.get_image_extractor()
 
-        # TODO: use the log prefix
-        self.log_prefix = "crawler: "
-
     def crawl(self, crawl_candidate):
 
         # parser candidate
@@ -130,11 +128,12 @@ class Crawler(object):
         raw_html = self.get_html(crawl_candidate, parse_candidate)
 
         if raw_html is None:
+            logger.warning("No raw_html is provided or could be fetched; continuing with an empty Article object")
             return self.article
 
         return self.process(raw_html, parse_candidate.url, parse_candidate.link_hash)
 
-    def process(self, raw_html, final_url, link_hash):
+    def process(self, raw_html: str, final_url: str, link_hash: str) -> Article:
 
         # create document
         doc = self.get_document(raw_html)
@@ -164,25 +163,17 @@ class Crawler(object):
         # meta
         metas = self.metas_extractor.extract()
         # print(metas)
-        self.article._meta_lang = metas['lang']
-        self.article._meta_favicon = metas['favicon']
-        self.article._meta_description = metas['description']
-        self.article._meta_keywords = metas['keywords']
-        self.article._meta_encoding = metas['encoding']
-        self.article._canonical_link = metas['canonical']
-        self.article._domain = metas['domain']
+        self.article._meta_lang = metas["lang"]
+        self.article._meta_favicon = metas["favicon"]
+        self.article._meta_description = metas["description"]
+        self.article._meta_keywords = metas["keywords"]
+        self.article._meta_encoding = metas["encoding"]
+        self.article._canonical_link = metas["canonical"]
+        self.article._domain = metas["domain"]
 
         # publishdate
         self.article._publish_date = self.publishdate_extractor.extract()
-        if self.article.publish_date:
-            try:
-                publish_datetime = dateutil.parser.parse(self.article.publish_date, tzinfos=TIMEZONE_INFO)
-                if publish_datetime.tzinfo:
-                    self.article._publish_datetime_utc = publish_datetime.astimezone(tzutc())
-                else:
-                    self.article._publish_datetime_utc = publish_datetime
-            except (ValueError, OverflowError):
-                self.article._publish_datetime_utc = None
+        self.article._publish_datetime_utc = self._publish_date_to_utc() if self.article.publish_date else None
 
         # tags
         self.article._tags = self.tags_extractor.extract()
@@ -195,17 +186,7 @@ class Crawler(object):
 
         # jump through some hoops on attempting to get a language if not found
         if self.article._meta_lang is None:
-            tmp_lang_detect = "{} {} {} {}".format(self.article._meta_description, self.article._title, self.article._meta_keywords, self.article._tags)
-            tmp_lang_detect = " ".join(tmp_lang_detect.split())
-            if len(tmp_lang_detect) > 15:
-                # required to make it deterministic;
-                # see: https://github.com/Mimino666/langdetect/blob/master/README.md#basic-usage
-                DetectorFactory.seed = 0
-                try:
-                    self.article._meta_lang = detect(tmp_lang_detect)
-                except LangDetectException:
-                    self.article._meta_lang = None
-            # print(self.article._meta_lang)
+            self.article._meta_lang = self._alternative_language_extractor()
 
         # check for known node as content body
         # if we find one force the article.doc to be the found node
@@ -262,25 +243,26 @@ class Crawler(object):
         return self.article
 
     @staticmethod
-    def get_parse_candidate(crawl_candidate):
+    def get_parse_candidate(crawl_candidate: CrawlCandidate) -> ParsingCandidate:
         if crawl_candidate.raw_html:
             return RawHelper.get_parsing_candidate(crawl_candidate.url, crawl_candidate.raw_html)
         return URLHelper.get_parsing_candidate(crawl_candidate.url)
 
     def get_image(self):
-        doc = self.article.raw_doc
         top_node = self.article.top_node
-        self.article._top_image = self.image_extractor.get_best_image(doc, top_node)
+        self.article._top_image = self.image_extractor.get_best_image(top_node)
 
-    def get_html(self, crawl_candidate, parsing_candidate):
+    def get_html(self, crawl_candidate: CrawlCandidate, parsing_candidate: ParsingCandidate) -> str:
         # we got a raw_tml
         # no need to fetch remote content
         if crawl_candidate.raw_html:
+            logger.debug("Using raw_html for %s", crawl_candidate)
             return crawl_candidate.raw_html
 
         # fetch HTML
+        logger.debug("Fetching html from %s", crawl_candidate.url)
         response = self.fetcher.fetch_obj(parsing_candidate.url)
-        if response.encoding != 'ISO-8859-1':  # requests has a good idea; use what it says
+        if response.encoding != "ISO-8859-1":  # requests has a good idea; use what it says
             # return response as a unicode string
             html = response.text
             self.article._meta_encoding = response.encoding
@@ -342,10 +324,33 @@ class Crawler(object):
         return StandardContentExtractor(self.config, self.article)
 
     def release_resources(self):
-        path = os.path.join(self.config.local_storage_path, '%s_*' % self.article.link_hash)
+        path = os.path.join(self.config.local_storage_path, f"{self.article.link_hash}_*")
         for fname in glob.glob(path):
             try:
                 os.remove(fname)
             except OSError:
-                # TODO: better log handeling
-                pass
+                logger.error("File %s could not be removed", fname)
+
+    def _publish_date_to_utc(self):
+        try:
+            publish_datetime = dateutil.parser.parse(self.article.publish_date, tzinfos=TIMEZONE_INFO)
+            if publish_datetime.tzinfo:
+                return publish_datetime.astimezone(tzutc())
+            return publish_datetime
+        except (ValueError, OverflowError, TypeError):
+            logger.warning("Publish date %s could not be resolved to UTC", self.article.publish_date)
+            return None
+
+    def _alternative_language_extractor(self):
+        art = self.article
+        tmp_lang_detect = f"{art.meta_description} {art.title} {art.meta_keywords} {art.tags}"
+        tmp_lang_detect = " ".join(tmp_lang_detect.split())
+        if len(tmp_lang_detect) > 15:
+            # required to make it deterministic;
+            # see: https://github.com/Mimino666/langdetect/blob/master/README.md#basic-usage
+            DetectorFactory.seed = 0
+            try:
+                return detect(tmp_lang_detect)
+            except LangDetectException:
+                logger.warning("Alternative language extractor failed to extract a known language")
+        return None
