@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-"""\
+"""
 This is a python port of "Goose" orignialy licensed to Gravity.com
 under one or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
@@ -24,36 +23,32 @@ import re
 
 from goose3.extractors import BaseExtractor
 
-
 TITLE_SPLITTERS = ["|", "-", "»", ":"]
 
 
 class TitleExtractor(BaseExtractor):
-
     def clean_title(self, title):
-        """Clean title with the use of og:site_name
-        in this case try to get rid of site name
-        and use TITLE_SPLITTERS to reformat title
-        """
+        """Clean title with the use of og:site_name in this case try to get rid of site name and use
+        TITLE_SPLITTERS to reformat title"""
 
         # try to remove the domain from url
         if self.article.domain:
             pattern = re.compile(self.article.domain, re.IGNORECASE)
             title = pattern.sub("", title).strip()
 
+        schema = self.article.schema
         site_name = ""
         # check if we have the site name in opengraph data
-        if "site_name" in list(self.article.opengraph.keys()) and self.article.opengraph['site_name'] != title:
-            site_name = self.article.opengraph['site_name']
-        elif (self.article.schema and "publisher" in self.article.schema and "name" in self.article.schema["publisher"]):
+        if "site_name" in self.article.opengraph and self.article.opengraph["site_name"] != title:
+            site_name = self.article.opengraph["site_name"]
+        elif schema and schema.get("publisher") and schema["publisher"].get("name"):
             site_name = self.article.schema["publisher"]["name"]
 
-
         # if there is a sperator, speratate and check if site name is present
         seps = [s for s in TITLE_SPLITTERS if s in title]
         if seps:
             # NOTE: Perhaps it should only be removed if, once seperated it is "by itself"
-            title = title.replace(site_name, '').strip()
+            title = title.replace(site_name, "").strip()
 
         # split the title in words
         # TechCrunch | my wonderfull article
@@ -81,30 +76,25 @@ class TitleExtractor(BaseExtractor):
         return title
 
     def get_title(self):
-        """\
-        Fetch the article title and analyze it
-        """
-        title = ''
+        """Fetch the article title and analyze it"""
+        title = ""
 
         # rely on opengraph in case we have the data
-        if "title" in list(self.article.opengraph.keys()):
-            return self.clean_title(self.article.opengraph['title'])
-        elif self.article.schema and "headline" in self.article.schema:
-            return self.clean_title(self.article.schema['headline'])
+        if "title" in self.article.opengraph:
+            return self.clean_title(self.article.opengraph["title"])
+        if self.article.schema and "headline" in self.article.schema:
+            return self.clean_title(self.article.schema["headline"])
 
         # try to fetch the meta headline
-        meta_headline = self.parser.getElementsByTag(self.article.doc,
-                                                     tag="meta",
-                                                     attr="name",
-                                                     value="headline")
+        meta_headline = self.parser.get_elements_by_tag(self.article.doc, tag="meta", attr="name", value="headline")
         if meta_headline is not None and len(meta_headline) > 0:
-            title = self.parser.getAttribute(meta_headline[0], 'content')
+            title = self.parser.get_attribute(meta_headline[0], "content")
             return self.clean_title(title)
 
         # otherwise use the title meta
-        title_element = self.parser.getElementsByTag(self.article.doc, tag='title')
+        title_element = self.parser.get_elements_by_tag(self.article.doc, tag="title")
         if title_element is not None and len(title_element) > 0:
-            title = self.parser.getText(title_element[0])
+            title = self.parser.get_text(title_element[0])
             return self.clean_title(title)
 
         return title
