@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-"""\
+"""
 This is a python port of "Goose" orignialy licensed to Gravity.com
 under one or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
@@ -20,22 +19,19 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 """
-
 from goose3.extractors import BaseExtractor
 
 
 class TweetsExtractor(BaseExtractor):
-
     def extract(self):
         tweets = []
-        items = self.parser.getElementsByTag(self.article.top_node,
-                                             tag='blockquote',
-                                             attr="class",
-                                             value="twitter-tweet")
+        items = self.parser.get_elements_by_tag(
+            self.article.top_node, tag="blockquote", attr="class", value="twitter-tweet"
+        )
 
         for i in items:
-            for attr in ['gravityScore', 'gravityNodes']:
-                self.parser.delAttribute(i, attr)
-            tweets.append(self.parser.nodeToString(i))
+            for attr in ["gravityScore", "gravityNodes"]:
+                self.parser.del_attribute(i, attr)
+            tweets.append(self.parser.node_to_string(i))
 
         return tweets
