@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-"""\
+"""
 This is a python port of "Goose" orignialy licensed to Gravity.com
 under one or more contributor license agreements.  See the NOTICE file
 distributed with this work for additional information
@@ -23,53 +22,41 @@ limitations under the License.
 import os
 import re
 import string
+import warnings
 
-from goose3.utils import FileHelper
-from goose3.utils.encoding import (smart_unicode, smart_str, DjangoUnicodeDecodeError)
+from goose3.utils import FileHelper, deprecated
+from goose3.utils.constants import CAMEL_CASE_DEPRICATION
+from goose3.utils.encoding import DjangoUnicodeDecodeError, smart_str, smart_unicode
 
-
-SPACE_SYMBOLS = re.compile(r'[\s\xa0\t]')
-TABSSPACE = re.compile(r'[\s\t]+')
+SPACE_SYMBOLS = re.compile(r"[\s\xa0\t]")
+TABSSPACE = re.compile(r"[\s\t]+")
 
 
 def get_encodings_from_content(content):
-    """
-    Code from:
-    https://github.com/sigmavirus24/requests-toolbelt/blob/master/requests_toolbelt/utils/deprecated.py
+    """Code from: https://github.com/sigmavirus24/requests-toolbelt/blob/master/requests_toolbelt/utils/deprecated.py
     Return encodings from given content string.
-    :param content: string to extract encodings from.
-    """
+    :param content: string to extract encodings from."""
     if isinstance(content, bytes):
-        find_charset = re.compile(
-            br'<meta.*?charset=["\']*([a-zA-Z0-9\-_]+?) *?["\'>]', flags=re.I
-        ).findall
+        content = content.decode()
 
-        find_xml = re.compile(
-            br'^<\?xml.*?encoding=["\']*([a-zA-Z0-9\-_]+?) *?["\'>]'
-        ).findall
-        return [encoding.decode('utf-8') for encoding in
-                find_charset(content) + find_xml(content)]
-    else:
-        find_charset = re.compile(
-            r'<meta.*?charset=["\']*([a-zA-Z0-9\-_]+?) *?["\'>]', flags=re.I
-        ).findall
+    find_charset = re.compile(
+        r'<meta.*?charset=["\']*[^a-zA-z0-9]*([a-zA-Z0-9\-_]+?)[^a-zA-z0-9]* *?["\'>]', flags=re.I
+    ).findall
 
-        find_xml = re.compile(
-            r'^<\?xml.*?encoding=["\']*([a-zA-Z0-9\-_]+?) *?["\'>]'
-        ).findall
-        return find_charset(content) + find_xml(content)
+    find_xml = re.compile(r'^<\?xml.*?encoding=["\']*([a-zA-Z0-9\-_]+?) *?["\'>]').findall
+    return find_charset(content) + find_xml(content)
 
 
-def innerTrim(value):
+def inner_trim(value):
     if isinstance(value, str):
         # remove tab and white space
-        value = re.sub(TABSSPACE, ' ', value)
-        value = ''.join(value.splitlines())
+        value = re.sub(TABSSPACE, " ", value)
+        value = "".join(value.splitlines())
         return value.strip()
-    return ''
+    return ""
 
 
-def encodeValue(value):
+def encode_value(value):
     string_org = value
     try:
         value = smart_unicode(value)
@@ -80,8 +67,18 @@ def encodeValue(value):
     return value
 
 
-class WordStats(object):
+# Aliases
+@deprecated(f"Deprecated and to be removed in v{CAMEL_CASE_DEPRICATION}; use inner_trim instead")
+def innerTrim(value):
+    return inner_trim(value)
 
+
+@deprecated(f"Deprecated and to be removed in v{CAMEL_CASE_DEPRICATION}; use encode_value instead")
+def encodeValue(value):
+    return encode_value(value)
+
+
+class WordStats:
     def __init__(self):
         # total number of stopwords or
         # good words that we can calculate
@@ -113,16 +110,16 @@ class WordStats(object):
         self.word_count = cnt
 
 
-class StopWords(object):
+class StopWords:
     _cached_stop_words = {}
 
-    def __init__(self, language='en'):
+    def __init__(self, language="en"):
         if language not in self._cached_stop_words:
-            path = os.path.join('text', 'stopwords-%s.txt' % language)
+            path = os.path.join("resources", "text", f"stopwords-{language}.txt")
             try:
-                content = FileHelper.loadResourceFile(path)
+                content = FileHelper.load_resource_file(path)
                 word_list = content.splitlines()
-            except IOError:
+            except OSError:
                 word_list = []
             self._cached_stop_words[language] = set(word_list)
         self._stop_words = self._cached_stop_words[language]
@@ -132,7 +129,7 @@ class StopWords(object):
         # code taken form
         # http://stackoverflow.com/questions/265960/best-way-to-strip-punctuation-from-a-string-in-python
         if not isinstance(content, str):
-            content = content.decode('utf-8')
+            content = content.decode("utf-8")
         tbl = dict.fromkeys(ord(x) for x in string.punctuation)
         return content.translate(tbl)
 
@@ -160,29 +157,31 @@ class StopWords(object):
 
 
 class StopWordsChinese(StopWords):
-    """
-    Chinese segmentation
-    """
-    def __init__(self, language='zh'):
+    """Chinese segmentation"""
+
+    def __init__(self, language="zh"):
         # force zh languahe code
-        super(StopWordsChinese, self).__init__(language='zh')
+        super().__init__(language="zh")
 
     @staticmethod
     def candidate_words(stripped_input):
         # jieba build a tree that takes sometime
         # avoid building the tree if we don't use
         # chinese language
-        import jieba
+        try:
+            import jieba
+        except ImportError:
+            warnings.warn("jieba is not installed. To use Chinese, one must install the jieba package")
+            return []
         return jieba.cut(stripped_input, cut_all=True)
 
 
 class StopWordsArabic(StopWords):
-    """
-    Arabic segmentation
-    """
-    def __init__(self, language='ar'):
+    """Arabic segmentation"""
+
+    def __init__(self, language="ar"):
         # force ar languahe code
-        super(StopWordsArabic, self).__init__(language='ar')
+        super().__init__(language="ar")
 
     @staticmethod
     def remove_punctuation(content):
@@ -190,7 +189,11 @@ class StopWordsArabic(StopWords):
 
     @staticmethod
     def candidate_words(stripped_input):
-        import nltk
+        try:
+            import nltk
+        except ImportError:
+            warnings.warn("NLTK is not installed. To use Arabic, one must install the nltk package")
+            return []
         stemmer = nltk.stem.isri.ISRIStemmer()
         words = []
         for word in nltk.tokenize.wordpunct_tokenize(stripped_input):
@@ -199,11 +202,18 @@ class StopWordsArabic(StopWords):
 
 
 class StopWordsKorean(StopWords):
-    """
-    Korean segmentation
-    """
-    def __init__(self, language='ko'):
-        super(StopWordsKorean, self).__init__(language='ko')
+    """Korean segmentation"""
+
+    def __init__(self, language="ko"):
+        super().__init__(language="ko")
+        # Korean StopWords are attached at noun without a space
+        # To find the stopwords in given sentences quickly, Ahocorasick is needed
+        import ahocorasick
+
+        self.auto = ahocorasick.Automaton()
+        for word in self._stop_words:
+            self.auto.add_word(word, word)
+        self.auto.make_automaton()
 
     def get_stopword_count(self, content):
         if not content:
@@ -212,13 +222,10 @@ class StopWordsKorean(StopWords):
         stripped_input = self.remove_punctuation(content)
         candidate_words = self.candidate_words(stripped_input)
         overlapping_stopwords = []
-        i = 0
-        for _ in candidate_words:
-            i += 1
-            for stop_word in self._stop_words:
-                overlapping_stopwords.append(stop_word)
+        for item in self.auto.iter("".join(candidate_words)):
+            overlapping_stopwords.append(item[1])
 
-        stats.set_word_count(i)
+        stats.set_word_count(len(candidate_words))
         stats.set_stopword_count(len(overlapping_stopwords))
         stats.set_stop_words(overlapping_stopwords)
         return stats
